@@ -1,5 +1,5 @@
 import os
-from typing import Optional, Literal, Union
+from typing import Optional, Union
 import scanpy as sc
 from anndata import AnnData
 import numpy as np
@@ -39,7 +39,7 @@ class AnnotationResult():
     cell_count
         Number of input cells which undergo the prediction process.
     adata
-        A Scanpy object representing the input data.
+        An :class:`~anndata.AnnData` object representing the input data.
     """
     def __init__(self, labels: pd.DataFrame, decision_mat: pd.DataFrame, prob_mat: pd.DataFrame, adata: AnnData):
         self.predicted_labels = labels
@@ -48,7 +48,7 @@ class AnnotationResult():
         self.adata = adata
         self.cell_count = labels.shape[0]
 
-    def summary_frequency(self, by: Literal['predicted_labels', 'majority_voting'] = 'predicted_labels') -> pd.DataFrame:
+    def summary_frequency(self, by: str = 'predicted_labels') -> pd.DataFrame:
         """
         Get the frequency of cells belonging to each cell type predicted by celltypist.
 
@@ -69,20 +69,20 @@ class AnnotationResult():
         df.sort_values(['counts'], ascending=False, inplace=True)
         return df
 
-    def to_adata(self, insert_labels: bool = True, insert_decision: bool = False, insert_probability: bool = False) -> AnnData:
+    def to_adata(self, insert_labels: bool = True, insert_decision: bool = False, insert_prob: bool = False) -> AnnData:
         """
-        Insert the predicted labels, decision or probability matrix, and (if majority voting is done) majority voting results into the Scanpy object.
+        Insert the predicted labels, decision or probability matrix, and (if majority voting is done) majority voting results into the AnnData object.
 
         Parameters
         ----------
         insert_labels
-            Whether to insert the predicted cell type labels and (if majority voting is done) majority voting-based labels into the Scanpy object.
+            Whether to insert the predicted cell type labels and (if majority voting is done) majority voting-based labels into the AnnData object.
             (Default: `True`)
         insert_decision
-            Whether to insert the decision matrix into the Scanpy object.
+            Whether to insert the decision matrix into the AnnData object.
             (Default: `False`)
-        insert_probability
-            Whether to insert the probability matrix into the Scanpy object. This will override the decision matrix even when `insert_decision` is set to `True`.
+        insert_prob
+            Whether to insert the probability matrix into the AnnData object. This will override the decision matrix even when `insert_decision` is set to `True`.
             (Default: `False`)
 
         Returns
@@ -92,11 +92,11 @@ class AnnotationResult():
             1) **predicted_labels**, individual prediction outcome for each cell.
             2) **over_clustering**, over-clustering result for the cells.
             3) **majority_voting**, the cell type label assigned to each cell after the majority voting process.
-            4) **name of each cell type**, which represents the decision scores (or probabilities if `insert_probability` is `True`) of a given cell type across cells.
+            4) **name of each cell type**, which represents the decision scores (or probabilities if `insert_prob` is `True`) of a given cell type across cells.
         """
         if insert_labels:
             self.adata.obs[self.predicted_labels.columns] = self.predicted_labels
-        if insert_probability:
+        if insert_prob:
             self.adata.obs[self.probability_matrix.columns] = self.probability_matrix
         elif insert_decision:
             self.adata.obs[self.decision_matrix.columns] = self.decision_matrix
@@ -196,12 +196,12 @@ class Classifier():
     Parameters
     ----------
     filename
-        Path to the input count matrix (supported types are csv, txt, tsv, tab and mtx) or Scanpy object (h5ad).
+        Path to the input count matrix (supported types are csv, txt, tsv, tab and mtx) or AnnData object (h5ad).
         If it's the former, a cell-by-gene format is desirable (see `transpose` for more information).
         Also accepts the input as an :class:`~anndata.AnnData` object already loaded in memory.
         Genes should be gene symbols. Non-expressed genes are preferred to be provided as well.
     model
-        A :class:`~celltypist.models.Model` object that wraps the SGDClassifier and the StandardScaler, the
+        A :class:`~celltypist.models.Model` object that wraps the logistic Classifier and the StandardScaler, the
         path to the desired model file, or the model name.
     transpose
         Whether to transpose the input matrix. Set to `True` if `filename` is provided in a gene-by-cell format.
@@ -218,7 +218,7 @@ class Classifier():
     filename
         Path to the input dataset. This attribute exists only when the input is a file path.
     adata
-        A Scanpy object which stores the log1p normalized expression data in `.X` or `.raw.X`.
+        An :class:`~anndata.AnnData` object which stores the log1p normalized expression data in `.X` or `.raw.X`.
     indata
         The expression matrix used for predictions stored in the log1p normalized format.
     indata_genes
@@ -226,7 +226,7 @@ class Classifier():
     indata_names
         All the cells included in the input data.
     model
-        A :class:`~celltypist.models.Model` object that wraps the SGDClassifier and the StandardScaler.
+        A :class:`~celltypist.models.Model` object that wraps the logistic Classifier and the StandardScaler.
     """
     def __init__(self, filename: Union[AnnData,str] = "", model: Union[Model,str] = "", transpose: bool = False, gene_file: Optional[str] = None, cell_file: Optional[str] = None):
         if isinstance(model, str):
@@ -255,6 +255,10 @@ class Classifier():
                 self.adata.var_names = genes_mtx
                 self.adata.obs_names = cells_mtx
             self.adata.var_names_make_unique()
+            if not float(self.adata.X.max()).is_integer():
+                logger.warn(f"⚠️ Warning: the input file seems not a raw count matrix. The prediction result may be biased")
+            if (self.adata.n_vars >= 80000) or (len(self.adata.var_names[0]) >= 30) or (len(self.adata.obs_names.intersection(pd.Index(['GAPDH', 'ACTB', 'CALM1']))) >= 1):
+                raise ValueError(f"🛑 The input matrix is detected to be a gene-by-cell matrix. Please provide a cell-by-gene matrix or add the input transpose option")
             sc.pp.normalize_total(self.adata, target_sum=1e4)
             sc.pp.log1p(self.adata)
             self.indata = self.adata.X.copy()
@@ -262,6 +266,7 @@ class Classifier():
             self.indata_names = self.adata.obs_names.copy()
         elif isinstance(filename, AnnData) or (isinstance(filename, str) and filename.endswith('.h5ad')):
             self.adata = sc.read(filename) if isinstance(filename, str) else filename
+            self.adata.var_names_make_unique()
             if self.adata.X.min() < 0:
                 logger.info("👀 Detected scaled expression in the input data, will try the .raw attribute")
                 try:
@@ -281,10 +286,21 @@ class Classifier():
 
         logger.info(f"🔬 Input data has {self.indata.shape[0]} cells and {len(self.indata_genes)} genes")
 
-    def celltype(self) -> AnnotationResult:
+    def celltype(self, mode: str = 'best match', p_thres: float = 0.5) -> AnnotationResult:
         """
         Run celltyping jobs to predict cell types of input data.
 
+        Parameters
+        ----------
+        mode
+            The way cell prediction is performed.
+            For each query cell, the default (`best match`) is to choose the cell type with the largest score/probability as the final prediction.
+            Setting to `prob match` will enable a multi-label classification, which assigns 0 (i.e., unassigned), 1, or >=2 cell type labels to each query cell.
+            (Default: `best match`)
+        p_thres
+            Probability threshold for the multi-label classification. Ignored if `mode` is `best match`.
+            (Default: 0.5)
+
         Returns
         ----------
         :class:`~celltypist.classifier.AnnotationResult`
@@ -292,7 +308,7 @@ class Classifier():
             1) :attr:`~celltypist.classifier.AnnotationResult.predicted_labels`, predicted labels from celltypist.
             2) :attr:`~celltypist.classifier.AnnotationResult.decision_matrix`, decision matrix from celltypist.
             3) :attr:`~celltypist.classifier.AnnotationResult.probability_matrix`, probability matrix from celltypist.
-            4) :attr:`~celltypist.classifier.AnnotationResult.adata`, Scanpy object representation of the input data.
+            4) :attr:`~celltypist.classifier.AnnotationResult.adata`, AnnData object representation of the input data.
         """
         logger.info(f"🔗 Matching reference genes in the model")
         k_x = np.isin(self.indata_genes, self.model.classifier.features)
@@ -314,7 +330,7 @@ class Classifier():
         self.model.classifier.coef_ = self.model.classifier.coef_[:, lr_idx]
 
         logger.info("🖋️ Predicting labels")
-        decision_mat, prob_mat, lab = self.model.predict_labels_and_prob(self.indata)
+        decision_mat, prob_mat, lab = self.model.predict_labels_and_prob(self.indata, mode = mode, p_thres = p_thres)
         logger.info("✅ Prediction done!")
 
         cells = self.indata_names
@@ -323,13 +339,15 @@ class Classifier():
     @staticmethod
     def _construct_neighbor_graph(adata: AnnData) -> tuple:
         """Construct a neighborhood graph. This function is for internal use."""
-        if adata.X.min() < 0:
-            adata = adata.raw.to_adata()
-        sc.pp.filter_genes(adata, min_cells=5)
-        sc.pp.highly_variable_genes(adata)
-        adata = adata[:, adata.var.highly_variable]
-        sc.pp.scale(adata, max_value=10)
-        sc.tl.pca(adata, n_comps=50)
+        if 'X_pca' not in adata.obsm.keys():
+            if adata.X.min() < 0:
+                adata = adata.raw.to_adata()
+            if 'highly_variable' not in adata.var:
+                sc.pp.filter_genes(adata, min_cells=5)
+                sc.pp.highly_variable_genes(adata, n_top_genes = min([2500, adata.n_vars]))
+            adata = adata[:, adata.var.highly_variable]
+            sc.pp.scale(adata, max_value=10)
+            sc.tl.pca(adata, n_comps=50)
         sc.pp.neighbors(adata, n_neighbors=10, n_pcs=50)
         return adata.obsm['X_pca'], adata.obsp['connectivities'], adata.obsp['distances'], adata.uns['neighbors']
 
@@ -370,7 +388,7 @@ class Classifier():
         return oc_column
 
     @staticmethod
-    def majority_vote(predictions: AnnotationResult, over_clustering: Union[list, tuple, np.ndarray, pd.Series, pd.Index]) -> AnnotationResult:
+    def majority_vote(predictions: AnnotationResult, over_clustering: Union[list, tuple, np.ndarray, pd.Series, pd.Index], min_prop: float = 0) -> AnnotationResult:
         """
         Majority vote the celltypist predictions using the result from the over-clustering.
 
@@ -380,6 +398,9 @@ class Classifier():
             An :class:`~celltypist.classifier.AnnotationResult` object containing the :attr:`~celltypist.classifier.AnnotationResult.predicted_labels`.
         over_clustering
             A list, tuple, numpy array, pandas series or index containing the over-clustering information.
+        min_prop
+            For the dominant cell type within a subcluster, the minimum proportion of cells required to support naming of the subcluster by this cell type.
+            (Default: 0)
 
         Returns
         ----------
@@ -388,13 +409,16 @@ class Classifier():
             1) :attr:`~celltypist.classifier.AnnotationResult.predicted_labels`, predicted labels from celltypist.
             2) :attr:`~celltypist.classifier.AnnotationResult.decision_matrix`, decision matrix from celltypist.
             3) :attr:`~celltypist.classifier.AnnotationResult.probability_matrix`, probability matrix from celltypist.
-            4) :attr:`~celltypist.classifier.AnnotationResult.adata`, Scanpy object representation of the input data.
+            4) :attr:`~celltypist.classifier.AnnotationResult.adata`, AnnData object representation of the input data.
         """
         if isinstance(over_clustering, (list, tuple)):
             over_clustering = np.array(over_clustering)
         logger.info("🗳️ Majority voting the predictions")
         votes = pd.crosstab(predictions.predicted_labels['predicted_labels'], over_clustering)
-        majority = votes.idxmax()[over_clustering].reset_index()
+        majority = votes.idxmax(axis=0)
+        freqs = (votes / votes.sum(axis=0).values).max(axis=0)
+        majority[freqs < min_prop] = 'Heterogeneous'
+        majority = majority[over_clustering].reset_index()
         majority.index = predictions.predicted_labels.index
         majority.columns = ['over_clustering', 'majority_voting']
         majority['majority_voting'] = majority['majority_voting'].astype('category')
